@@ -16,10 +16,11 @@ class JsRoutes
     include: //,
     file: DEFAULT_PATH,
     prefix: nil,
-    url_links: nil,
+    url_links: false,
     camel_case: false,
     default_url_options: {},
-    compact: false
+    compact: false,
+    serializer: nil
   }
 
   NODE_TYPES = {
@@ -34,6 +35,7 @@ class JsRoutes
   }
 
   LAST_OPTIONS_KEY = "options".freeze
+  FILTERED_DEFAULT_PARTS = [:controller, :action]
 
   class Options < Struct.new(*DEFAULTS.keys)
     def to_hash
@@ -57,6 +59,11 @@ class JsRoutes
     end
 
     def generate(opts = {})
+      # Ensure routes are loaded. If they're not, load them.
+      if Rails.application.routes.named_routes.to_a.empty?
+        Rails.application.reload_routes!
+      end
+
       new(opts).generate
     end
 
@@ -92,21 +99,40 @@ class JsRoutes
   end
 
   def generate
-    js = File.read(File.dirname(__FILE__) + "/routes.js")
-    js.gsub!("NAMESPACE", @options[:namespace])
-    js.gsub!("DEFAULT_URL_OPTIONS", json(@options[:default_url_options].merge(deprecated_default_format)))
-    js.gsub!("PREFIX", @options[:prefix] || "")
-    js.gsub!("NODE_TYPES", json(NODE_TYPES))
-    js.gsub!("ROUTES", js_routes)
+    {
+      "GEM_VERSION"         => JsRoutes::VERSION,
+      "APP_CLASS"           => Rails.application.class.to_s,
+      "NAMESPACE"           => @options[:namespace],
+      "DEFAULT_URL_OPTIONS" => json(@options[:default_url_options].merge(deprecate_url_options)),
+      "PREFIX"              => @options[:prefix] || "",
+      "NODE_TYPES"          => json(NODE_TYPES),
+      "SERIALIZER"          => @options[:serializer] || "null",
+      "ROUTES"              => js_routes,
+    }.inject(File.read(File.dirname(__FILE__) + "/routes.js")) do |js, (key, value)|
+      js.gsub!(key, value)
+    end
   end
 
-  def deprecated_default_format
+  def deprecate_url_options
+    result = {}
     if @options.key?(:default_format)
       warn("default_format option is deprecated. Use default_url_options = { format: <format> } instead")
-      { format: @options[:default_format] }
-    else
-      {}
+      result.merge!(  format: @options[:default_format]  )
     end
+    if @options[:url_links].is_a?(String)
+      ActiveSupport::Deprecation.warn('js-routes url_links config value must be a boolean. Use default_url_options for specifying a default host.')
+
+      raise "invalid URL format in url_links (ex: http[s]://example.com)" if @options[:url_links].match(URI::Parser.new.make_regexp(%w(http https))).nil?
+      uri = URI.parse(@options[:url_links])
+      default_port = uri.scheme == "https" ? 443 : 80
+      port = uri.port == default_port ? nil : uri.port
+      result.merge!(
+        host: uri.host,
+        port: port,
+        protocol: uri.scheme,
+      )
+    end
+    result
   end
 
   def generate!(file_name = nil)
@@ -124,7 +150,7 @@ class JsRoutes
   protected
 
   def js_routes
-    js_routes = Rails.application.routes.named_routes.routes.sort_by(&:to_s).map do |_, route|
+    js_routes = Rails.application.routes.named_routes.to_a.sort_by(&:to_s).flat_map do |_, route|
       rails_engine_app = get_app_from_route(route)
       if rails_engine_app.respond_to?(:superclass) && rails_engine_app.superclass == Rails::Engine && !route.path.anchored
         rails_engine_app.routes.named_routes.map do |_, engine_route|
@@ -133,7 +159,7 @@ class JsRoutes
       else
         build_route_if_match(route)
       end
-    end.flatten.compact
+    end.compact
 
     "{\n" + js_routes.join(",\n") + "}\n"
   end
@@ -156,51 +182,43 @@ class JsRoutes
   end
 
   def any_match?(route, parent_route, matchers)
+    full_route = [parent_route.try(:name), route.name].compact.join('_')
+
     matchers = Array(matchers)
-    matchers.any? {|regex| [parent_route.try(:name), route.name].compact.join('_') =~ regex}
+    matchers.any? {|regex| full_route =~ regex}
   end
 
   def build_js(route, parent_route)
     name = [parent_route.try(:name), route.name].compact
-    parent_spec = parent_route.try(:path).try(:spec)
-    required_parts, optional_parts = route.required_parts.clone, route.optional_parts.clone
-    optional_parts.push(required_parts.delete :format) if required_parts.include?(:format)
     route_name = generate_route_name(name, (:path unless @options[:compact]))
-    url_link = generate_url_link(name, route_name, required_parts, route)
+    parent_spec = parent_route.try(:path).try(:spec)
+    route_arguments = route_js_arguments(route, parent_spec)
+    url_link = generate_url_link(name, route_name, route_arguments, route)
     _ = <<-JS.strip!
   // #{name.join('.')} => #{parent_spec}#{route.path.spec}
-  // function(#{[required_parts, LAST_OPTIONS_KEY].flatten.join(', ')})
-  #{route_name}: Utils.route(#{json(required_parts)}, #{json(optional_parts)}, #{json(serialize(route.path.spec, parent_spec))}, arguments)#{",\n" + url_link if url_link.length > 0}
+  // function(#{build_params(route.required_parts)})
+  #{route_name}: Utils.route(#{route_arguments})#{",\n" + url_link if url_link.length > 0}
   JS
   end
 
-  def generate_url_link(name, route_name, required_parts, route)
-    return "" unless @options[:url_links]
-    _ = <<-JS.strip!
-    #{generate_route_name(name, :url)}: function(#{build_params(required_parts)}) {
-    return #{generate_base_url_js(route)} + this.#{route_name}(#{build_params(required_parts)});
-    }
-    JS
+  def route_js_arguments(route, parent_spec)
+    required_parts = route.required_parts.clone
+    optional_parts = route.parts - required_parts
+    default_parts = route.defaults.reject do |part, _|
+      FILTERED_DEFAULT_PARTS.include?(part)
+    end
+    [
+      required_parts, optional_parts, serialize(route.path.spec, parent_spec), default_parts
+    ].map do |argument|
+      json(argument)
+    end.join(", ")
   end
 
-  def generate_base_url_js(route)
-    # preserve and deprecate previous behavior
-    unless @options[:url_links] == true
-      ActiveSupport::Deprecation.warn('js-routes url_links config value must be a boolean. Use default_url_options for specifying a default host.')
-      raise "invalid URL format in url_links (ex: http[s]://example.com)" if @options[:url_links].match(URI::Parser.new.make_regexp(%w(http https))).nil?
-      return "#{@options[:url_links].inspect}"
-    else
-      protocol = route.defaults[:protocol] || @options[:default_url_options][:protocol] || 'http'
-      hostname = route.defaults[:host] || @options[:default_url_options][:host]
-      port = route.defaults[:port] || (@options[:default_url_options][:port] unless route.defaults[:host])
-      port = ":#{port}" if port
-
-      unless hostname
-        raise "A :default_url_options[:host] must be configured in order to generate *_url helpers"
-      end
-
-      return %Q|'#{protocol}://#{hostname}#{port}'|
-    end
+  def generate_url_link(name, route_name, route_arguments, route)
+    return "" unless @options[:url_links]
+    <<-JS.strip!
+    #{generate_route_name(name, :url)}: Utils.route(#{route_arguments}, true)
+    JS
   end
 
   def generate_route_name(name, suffix)
@@ -214,11 +232,7 @@ class JsRoutes
   end
 
   def build_params(required_parts)
-    params = required_parts.map do |name|
-      # prepending each parameter name with underscore
-      # to prevent conflict with JS reserved words
-      "_#{name}"
-    end << LAST_OPTIONS_KEY
+    params = required_parts + [LAST_OPTIONS_KEY]
     params.join(", ")
   end
 
