@@ -13,6 +13,30 @@ describe JsRoutes, "options" do
   let(:_options) { {} }
   let(:_warnings) { true }
 
+  context "when serializer is specified" do
+    # define custom serializer
+    # this is a nonsense serializer, which always returns foo=bar
+    # for all inputs
+    let(:_presetup){ %q(function myCustomSerializer(object, prefix) { return "foo=bar"; }) }
+    let(:_options) { {:serializer => "myCustomSerializer"} }
+
+    it "should set configurable serializer" do
+      # expect the nonsense serializer above to have appened foo=bar
+      # to the end of the path
+      expect(evaljs(%q(Routes.inboxes_path()))).to eql("/inboxes?foo=bar")
+    end
+  end
+
+  context "when serializer is specified, but not function" do
+    let(:_presetup){ %q(var myCustomSerializer = 1) }
+    let(:_options) { {:serializer => "myCustomSerializer"} }
+
+    it "should set configurable serializer" do
+      # expect to use default
+      expect(evaljs(%q(Routes.inboxes_path({a: 1})))).to eql("/inboxes?a=1")
+    end
+  end
+
   context "when exclude is specified" do
 
     let(:_options) { {:exclude => /^admin_/} }
@@ -114,15 +138,18 @@ describe JsRoutes, "options" do
       expect(evaljs("Routes.inbox_path(1, {format: null})")).to eq(routes.inbox_path(1))
     end
 
-    it "shouldn't include the format when {:format => false} is specified" do
-      expect(evaljs("Routes.no_format_path()")).to eq(routes.no_format_path)
-    end
 
     it "shouldn't require the format" do
+      pending if Rails.version < "4.0"
       expect(evaljs("Routes.json_only_path()")).to eq(routes.json_only_path(:format => 'json'))
     end
   end
 
+  it "shouldn't include the format when {:format => false} is specified" do
+    expect(evaljs("Routes.no_format_path()")).to eq(routes.no_format_path())
+    expect(evaljs("Routes.no_format_path({format: 'json'})")).to eq(routes.no_format_path(format: 'json'))
+  end
+
   describe "when namespace option is specified" do
     let(:_options) { {:namespace => "PHM"} }
     it "should use this namespace for routing" do
@@ -168,7 +195,6 @@ describe JsRoutes, "options" do
     context "with required route parts" do
       let(:_options) { {:default_url_options => {:inbox_id => "12"}} }
       it "should use this opions to fill optional parameters" do
-        pending
         expect(evaljs("Routes.inbox_messages_path()")).to eq(routes.inbox_messages_path(:inbox_id => 12))
       end
     end
@@ -284,19 +310,13 @@ describe JsRoutes, "options" do
     end
 
     context "when configuring with default_url_options" do
-      context "when default host is not specified" do
-        it "raises an error" do
-          expect { JsRoutes.generate({ :url_links => true }) }.to raise_error RuntimeError
-        end
-      end
-
       context "when only host option is specified" do
         let(:_options) { { :url_links => true, :default_url_options => {:host => "example.com"} } }
-        
+
         it "uses the specified host, defaults protocol to http, defaults port to 80 (leaving it blank)" do
           expect(evaljs("Routes.inbox_url(1)")).to eq("http://example.com#{routes.inbox_path(1)}")
         end
-        
+
         it "does not override protocol when specified in route" do
           expect(evaljs("Routes.new_session_url()")).to eq("https://example.com#{routes.new_session_path}")
         end
@@ -316,7 +336,7 @@ describe JsRoutes, "options" do
         it "uses the specified protocol and host, defaults port to 80 (leaving it blank)" do
           expect(evaljs("Routes.inbox_url(1)")).to eq("ftp://example.com#{routes.inbox_path(1)}")
         end
-        
+
         it "does not override protocol when specified in route" do
           expect(evaljs("Routes.new_session_url()")).to eq("https://example.com#{routes.new_session_path}")
         end
@@ -340,11 +360,11 @@ describe JsRoutes, "options" do
         it "does not override protocol when specified in route" do
           expect(evaljs("Routes.new_session_url()")).to eq("https://example.com:3000#{routes.new_session_path}")
         end
-        
+
         it "does not override host, protocol, or port when host is specified in route" do
-          expect(evaljs("Routes.sso_url()")).to eq(routes.sso_url)
+          expect(evaljs("Routes.sso_url()")).to eq("http://sso.example.com:3000" + routes.sso_path)
         end
-        
+
         it "does not override port when specified in route" do
           expect(evaljs("Routes.portals_url()")).to eq("http://example.com:8080#{routes.portals_path}")
         end
@@ -380,6 +400,54 @@ describe JsRoutes, "options" do
         end
       end
     end
+
+    context 'when window.location is present' do
+      let(:current_protocol) { 'http:' } # window.location.protocol includes the colon character
+      let(:current_hostname) { 'current.example.com' }
+      let(:current_port){ '' } # an empty string means port 80
+      let(:current_host) do
+        host = "#{current_hostname}"
+        host += ":#{current_port}" unless current_port == ''
+        host
+      end
+
+      before do
+        jscontext['window'] = {
+          :location => {
+            :protocol => current_protocol,
+            :hostname => current_hostname,
+            :port => current_port,
+            :host => current_host
+          }
+        }
+      end
+
+      context "without specifying a default host" do
+        let(:_options) { { :url_links => true } }
+
+        it "uses the current host" do
+          expect(evaljs("Routes.inbox_path")).not_to be_nil
+          expect(evaljs("Routes.inbox_url")).not_to be_nil
+          expect(evaljs("Routes.inbox_url(1)")).to eq("http://current.example.com#{routes.inbox_path(1)}")
+          expect(evaljs("Routes.inbox_url(1, { test_key: \"test_val\" })")).to eq("http://current.example.com#{routes.inbox_path(1, :test_key => "test_val")}")
+          expect(evaljs("Routes.new_session_url()")).to eq("https://current.example.com#{routes.new_session_path}")
+          expect(evaljs("Routes.sso_url()")).to eq("http://sso.example.com#{routes.sso_path}")
+
+        end
+
+        it "uses host option as an argument" do
+          expect(evaljs("Routes.portals_url({host: 'another.com'})")).to eq(routes.portals_url(host: 'another.com'))
+        end
+
+        it "uses port option as an argument" do
+          expect(evaljs("Routes.portals_url({host: 'localhost', port: 8080})")).to eq(routes.portals_url(host: 'localhost', port: 8080))
+        end
+
+        it "uses protocol option as an argument" do
+          expect(evaljs("Routes.portals_url({host: 'localhost', protocol: 'https'})")).to eq(routes.portals_url(protocol: 'https', host: 'localhost'))
+        end
+      end
+    end
   end
 
   describe "when the compact mode is enabled" do
@@ -397,7 +465,7 @@ describe JsRoutes, "options" do
             example.run
           end
         end
-        
+
         let(:_options) { { :compact => true, :url_links => "http://localhost" } }
         it "should not strip urls" do
           expect(evaljs("Routes.inbox(1)")).to eq(routes.inbox_path(1))
