@@ -54,6 +54,23 @@ describe JsRoutes, "compatibility with Rails"  do
     expect(evaljs("Routes.inbox_path(1, {expanded: true, anchor: 'hello'})")).to eq(routes.inbox_path(1, :expanded => true, :anchor => "hello"))
   end
 
+  it "should use irregular ActiveSupport pluralizations" do
+    expect(evaljs("Routes.budgies_path()")).to eq(routes.budgies_path)
+    expect(evaljs("Routes.budgie_path(1)")).to eq(routes.budgie_path(1))
+    expect(evaljs("Routes.budgy_path")).to eq(nil)
+    expect(evaljs("Routes.budgie_descendents_path(1)")).to eq(routes.budgie_descendents_path(1))
+  end
+
+  describe "when route has defaults" do
+    it "should support route default format" do
+      expect(evaljs("Routes.api_purchases_path()")).to eq(routes.api_purchases_path)
+    end
+
+    it "should support default format override" do
+      expect(evaljs("Routes.api_purchases_path({format: 'xml'})")).to eq(routes.api_purchases_path(format: 'xml'))
+    end
+  end
+
   context "with rails engines" do
     it "should support simple route" do
       expect(evaljs("Routes.blog_app_posts_path()")).to eq(blog_routes.posts_path())
@@ -74,6 +91,10 @@ describe JsRoutes, "compatibility with Rails"  do
     expect(evaljs("Routes.json_only_path({format: 'json'})")).to eq(routes.json_only_path(:format => 'json'))
   end
 
+  it "should serialize object with empty string value" do
+    expect(evaljs("Routes.inboxes_path({a: '', b: 1})")).to eq(routes.inboxes_path(:a => '', :b => 1))
+  end
+
   it "should support utf-8 route" do
     expect(evaljs("Routes.hello_path()")).to eq(routes.hello_path)
   end
@@ -141,48 +162,8 @@ describe JsRoutes, "compatibility with Rails"  do
       expect(evaljs("Routes.search_path({q: 'hello'})")).to eq(routes.search_path(:q => 'hello'))
     end
 
-    it "should ignore null parameters" do
-      pending
-      expect(evaljs("Routes.inboxes_path({hello: {world: null}})")).to eq(routes.inboxes_path(:hello => {world: nil}))
-    end
-  end
-
-  context "when jQuery is present" do
-    before do
-      evaljs("window.jQuery = {};")
-      jscontext[:parameterizeFunc] = lambda {|object| _value.to_param}
-      evaljs("window.jQuery.param = parameterizeFunc")
-    end
-
-    shared_examples_for "serialization" do
-      it "should support serialization of objects" do
-        expect(evaljs("window.jQuery.param(#{_value.to_json})")).to eq(_value.to_param)
-        expect(evaljs("Routes.inboxes_path(#{_value.to_json})")).to eq(routes.inboxes_path(_value))
-        expect(evaljs("Routes.inbox_path(1, #{_value.to_json})")).to eq(routes.inbox_path(1, _value))
-      end
-    end
-    context "when parameters is a hash" do
-      let(:_value) do
-        {:a => {:b => 'c'}, :q => [1,2]}
-      end
-      it_should_behave_like 'serialization'
-    end
-    context "when parameters is null" do
-      let(:_value) do
-        {:hello => {world: nil}}
-      end
-      it_should_behave_like 'serialization'
-    end
-    context "when parameters is null" do
-      let(:_value) do
-        nil
-      end
-
-      before do
-        pending("This test is invalid for nil/null and jruby #{JRUBY_VERSION}") if defined?(JRUBY_VERSION) && '1.7.13' == JRUBY_VERSION
-      end
-
-      it_should_behave_like 'serialization'
+    it "should support nested object null parameters" do
+      expect(evaljs("Routes.inboxes_path({hello: {world: null}})")).to eq(routes.inboxes_path(:hello => {:world => nil}))
     end
   end
 
@@ -204,17 +185,11 @@ describe JsRoutes, "compatibility with Rails"  do
         expect(evaljs("Routes.thing_path(null, 5)")).to eq(routes.thing_path(nil, 5))
       end
 
-      # Fail for rails 4.2. Reason: https://github.com/rails/rails/issues/12178
       it "should treat undefined as non-given optional part" do
-        # TODO: remove me after release of rails 4.2.1 (change Appraisals to "~> 4.2.1")
-        pending("This test is invalid for rails 4.2. Reason: https://github.com/rails/rails/issues/12178") if [4, 2, 0] == [Rails::VERSION::MAJOR, Rails::VERSION::MINOR, Rails::VERSION::TINY]
         expect(evaljs("Routes.thing_path(5, {optional_id: undefined})")).to eq(routes.thing_path(5, :optional_id => nil))
       end
 
-      # Fail for rails 4.2. Reason: https://github.com/rails/rails/issues/12178
       it "should treat null as non-given optional part" do
-        # TODO: remove me after release of rails 4.2.1 (change Appraisals to "~> 4.2.1")
-        pending("This test is invalid for rails 4.2. Reason: https://github.com/rails/rails/issues/12178") if [4, 2, 0] == [Rails::VERSION::MAJOR, Rails::VERSION::MINOR, Rails::VERSION::TINY]
         expect(evaljs("Routes.thing_path(5, {optional_id: null})")).to eq(routes.thing_path(5, :optional_id => nil))
       end
     end
@@ -224,6 +199,11 @@ describe JsRoutes, "compatibility with Rails"  do
         expect(evaljs("Routes.things_path({optional_id: 5})")).to eq(routes.things_path(:optional_id => 5))
       end
 
+      context "on nested optional parts" do
+        it "should include everything that is not optional" do
+          expect(evaljs("Routes.classic_path({controller: 'classic', action: 'edit'})")).to eq(routes.classic_path(controller: :classic, action: :edit))
+        end
+      end
     end
   end
 
@@ -321,7 +301,7 @@ describe JsRoutes, "compatibility with Rails"  do
     end
   end
 
-  context "when params" do
+  describe "required_params" do
     it "should show inbox spec" do
       expect(evaljs("Routes.inbox_path.required_params").to_a).to eq(["id"])
     end
@@ -330,4 +310,6 @@ describe JsRoutes, "compatibility with Rails"  do
       expect(evaljs("Routes.inbox_message_path.required_params").to_a).to eq(["inbox_id", "id"])
     end
   end
+
+
 end
