@@ -42,6 +42,10 @@ def blog_routes
   BlogEngine::Engine.routes.url_helpers
 end
 
+ActiveSupport::Inflector.inflections do |inflect|
+  inflect.irregular "budgie", "budgies"
+end
+
 
 module BlogEngine
   class Engine < Rails::Engine
@@ -62,6 +66,8 @@ class App < Rails::Application
   else
     config.paths['config/routes.rb'] << 'spec/config/routes.rb'
   end
+
+  config.root = File.expand_path('../dummy', __FILE__)
 end
 
 def draw_routes
@@ -95,6 +101,8 @@ def draw_routes
       resources :things
     end
 
+    get "/:controller(/:action(/:id))" => "classic#classic", :as => :classic
+
     get "/other_optional/(:optional_id)" => "foo#foo", :as => :foo
 
     get 'books/*section/:title' => 'books#show', :as => :book
@@ -108,12 +116,21 @@ def draw_routes
 
     get '/привет' => "foo#foo", :as => :hello
     get '(/o/:organization)/search/:q' => "foo#foo", as: :search
-
+      
     resources :sessions, :only => [:new, :create, :destroy], :protocol => 'https'
 
     get '/' => 'sso#login', host: 'sso.example.com', as: :sso
 
     resources :portals, :port => 8080
+
+    namespace :api, format: true, defaults: {format: 'json'} do
+      get "/purchases" => "purchases#index"
+    end
+
+    resources :budgies do
+      get "descendents"
+    end
+
   end
 
 end
@@ -133,7 +150,9 @@ RSpec.configure do |config|
   config.before(:all) do
     # compile all js files begin
     Dir["#{File.expand_path(File.join(File.dirname(__FILE__), "..", "lib"))}/**/*.coffee"].each do |coffee|
-      File.open(coffee.gsub(/\.coffee$/, ""), 'w') {|f| f.write(CoffeeScript.compile(File.read(coffee))) }
+      File.open(coffee.gsub(/\.coffee$/, ""), 'w') do |f|
+        f.write(CoffeeScript.compile(File.read(coffee)).lstrip)
+      end
     end
     # compile all js files end
     draw_routes
@@ -141,6 +160,22 @@ RSpec.configure do |config|
 
   config.before :each do
     evaljs("var window = this;", true)
-    jscontext[:log] = lambda {|context, value| puts value.inspect}
+
+    def inspectify(value)
+      case value
+      when V8::Object
+        value.to_h.map do |k,v|
+          [k, inspectify(v)]
+        end.to_h
+      when String, nil
+        value
+      else
+        raise "wtf #{value.class}?"
+      end
+
+    end
+    jscontext[:log] = lambda do |context, value|
+      puts inspectify(value).to_json
+    end
   end
 end
